@@ -5,13 +5,12 @@ pub mod kleptographic {
     pub use openssl::hash::{Hasher, MessageDigest};
     pub use rand::thread_rng;
     use rand::{AsByteSliceMut, Rng};
-    use serde::{Deserialize, Serialize};
     use sha3::{Digest, Keccak256};
     use std::env;
     use std::fs::File;
     use std::io::Write;
 
-    #[derive(Clone, Debug, Serialize, Deserialize)]
+    #[derive(Clone, Debug)]
     pub struct Param {
         a: Scalar<Secp256k1>,
         b: Scalar<Secp256k1>,
@@ -19,7 +18,7 @@ pub mod kleptographic {
         e: Scalar<Secp256k1>,
     }
 
-    #[derive(Clone, Debug, Serialize, Deserialize)]
+    #[derive(Clone, Debug)]
     pub struct Signature {
         pub r: Scalar<Secp256k1>,
         pub s: Scalar<Secp256k1>,
@@ -309,17 +308,35 @@ pub mod kleptographic {
 pub mod protocol {
     use crate::kleptographic::*;
     use curv::elliptic::curves::Scalar;
-    use serde::Deserialize;
-    use serde::Serialize;
+    use serde::{Deserialize, Serialize};
+    // use serde_json::Result;
     use sha3::digest::DynDigest;
     use std::io::{Read, Write};
+    use std::net::TcpStream;
     use std::os::unix::net::{UnixListener, UnixStream};
 
-    #[derive(Clone, Debug, Serialize, Deserialize)]
+    #[derive(Clone, Debug)]
     pub struct Packet {
         hash: Vec<u8>,
         sign: Signature,
     }
+    #[derive(Serialize, Deserialize, Debug)]
+    struct PacketMessage {
+        hash: String,
+        r: String,
+        s: String,
+        v: String,
+    }
+
+    impl PacketMessage {
+        pub fn from(packet: Packet) -> Self {
+            let hash = hex::encode(&packet.hash);
+            let r = packet.sign.r.to_bigint().to_hex();
+            let s = packet.sign.s.to_bigint().to_hex();
+            let v = packet.sign.v.to_hex();
+            PacketMessage { hash, r, s, v }
+        }
+    }
     impl Packet {
         pub fn from(hash: Vec<u8>, sign: Signature) -> Self {
             Packet { hash, sign }
@@ -349,15 +366,22 @@ pub mod protocol {
         )
     }
 
-    // send private key to socket. It will construct a transaction and send it
+    // send private key to socket. It will construct a normal transaction and send it
+    // on success this function will return hash of the transaction
     pub fn construct_transaction_and_send(
         private_key: String,
         stream: &mut UnixStream,
-    ) -> Result<usize, std::io::Error> {
-        let mut v = Vec::from(private_key.as_bytes());
+    ) -> Result<Vec<u8>, ()> {
+        let mut v = hex::decode(&private_key).unwrap();
         // the first 0 stands for construct_transaction_and_send mode -> "use this private key construct a normal transaction and send it for me"
         v.insert(0, 0);
-        stream.write(&v)
+        if let Ok(_) = stream.write(&v) {
+            let mut buf: [u8; 32] = [0; 32];
+            if let Ok(_) = stream.read_exact(&mut buf) {
+                return Ok(Vec::from(buf));
+            }
+        }
+        Err(())
     }
 
     // read hash from stream, sign it and send it back to socket
@@ -373,16 +397,59 @@ pub mod protocol {
 
         let sign = sign_hash(hash.clone(), keypair, Scalar::random());
         if let Some(sign) = sign {
-            Packet::from(hash, sign);
+            let packet_message = PacketMessage::from(Packet::from(hash, sign));
+            let message = serde_json::to_string(&packet_message).unwrap();
+            if let Ok(size) = stream.write(message.as_bytes()) {
+                return Ok(size);
+            } else {
+                return Err(());
+            }
         }
         Err(())
     }
+    // step 1: client construct a normal transaction and send it to the blockchain network
+    // then the client will send the transaction hash and sig(hash) to the server
+    pub fn client_step1(
+        keypair: KeyPair,
+        tcp_stream: &mut TcpStream,
+        sock_stream: &mut UnixStream,
+    ) -> Result<(), ()> {
+        let private = keypair.private.to_bigint().to_hex();
+        if let Ok(hash) = construct_transaction_and_send(private, sock_stream) {
+            if let Some(sign) = sign_hash(hash.clone(), keypair.clone(), Scalar::random()) {
+                let message = PacketMessage::from(Packet::from(hash, sign));
+                if let Ok(_) = tcp_stream.write(serde_json::to_string(&message).unwrap().as_bytes())
+                {
+                    return Ok(());
+                }
+            }
+        }
+        return Err(());
+    }
+    pub fn client_step2(
+        keypair: KeyPair,
+        tcp_stream: &mut TcpStream,
+        sock_stream: &mut UnixStream,
+    ) {
+    }
+    pub fn server_step1() {}
 }
 #[cfg(test)]
 mod tests {
     use crate::kleptographic::*;
     use sha3::{Digest, Keccak256};
 
+    #[test]
+    fn test_keypair() {
+        let private = Scalar::random();
+        let keypair = KeyPair::new(private.clone());
+        println!("{:?}", keypair.public);
+        let one: Scalar<Secp256k1> = Scalar::from(1);
+        let G = Point::generator() * one.clone();
+        println!("{:?}", G);
+        let G = G * one;
+        println!("{:?}", G);
+    }
     #[test]
     fn test_extract_users_private_key() {
         let message1 = String::from("first message");
